<?php

class alt_filtro extends WP_Widget
{

    function __construct()
    {
        parent::__construct(

        // Base ID of your widget
            'alt_filtro',

            // Widget name will appear in UI
            __('Altimus - Filtro de veículos', 'alt_filtro'),

            // Widget description
            array(
                'description' => __('Filtro lateral', 'Filiais'),
            )
        );
    }

    public function widget($args, $instance)
    {


        $versao = date('YmdHi');
        wp_enqueue_style('boostrap-slider-css', 'https://cdnjs.cloudflare.com/ajax/libs/bootstrap-slider/10.6.2/css/bootstrap-slider.min.css', array('master-css'), $versao, 'all');
        wp_enqueue_script('bootstrap-slider-js', 'https://cdnjs.cloudflare.com/ajax/libs/bootstrap-slider/10.6.2/bootstrap-slider.min.js', array(), $versao, true);
        wp_enqueue_style('widget-filtro', get_stylesheet_directory_uri() . '/widget/alt_filtro/assets/css/widget-filtro.css', '', $versao, 'all');
        wp_enqueue_style('widget-style', get_stylesheet_directory_uri() . '/widget/alt_filtro/assets/css/style.css', '', $versao, 'all');
        //wp_enqueue_script('angularjs', 'https://ajax.googleapis.com/ajax/libs/angularjs/1.7.8/angular.min.js', 'jquery', $versao, true);
        wp_enqueue_script('altimus', get_stylesheet_directory_uri() . '/widget/alt_filtro/assets/js/filtro.js', '', $versao, true);


        $title = apply_filters('widget_title', $instance['title']);
        $subtitle = $instance['subtitle'] ?? '';
        $number_of_posts = (!empty($instance['number_of_posts'])) ? $instance['number_of_posts'] : 1;
        $layout = isset($instance['layout']) ? esc_attr($instance['layout']) : 'block';

        $arrVeiculos = $this->listarVeiculos();


        $arrFiliais = $this->buscarFiliais($arrVeiculos);
        $arrAnoModelo = $this->buscarAnos($arrVeiculos);
        $arrMarcas = $this->buscarMarcas($arrVeiculos);

        $arrTipos = $this->buscarTipos($arrVeiculos);

        $arrModelos = $this->buscarModelos($arrVeiculos);
        $arrPrecos = $this->buscarPrecoMaximoEMinimo();
        $arrAnoMaximoMinimo = $this->buscarAnoMaximoEMinimo();

        ksort($arrMarcas);
        ksort($arrModelos);
        ksort($arrAnoModelo);

        $field_marca = $_GET['marca'] ?? null;
        $field_modelo = $_GET['modelo'] ?? null;
        $field_ano = $_GET['ano'] ?? null;
        $field_filial = $_GET['filial'] ?? null;

        $field_carroceria = $_GET['carroceria'] ?? null;

        $field_max_price = $_GET['max_price'] ?? $arrPrecos['maximo'];
        $field_min_price = $_GET['min_price'] ?? $arrPrecos['minimo'];


        $field_max_ano = $_GET['max_ano'] ?? $arrAnoMaximoMinimo['maximo'];
        $field_min_ano = $_GET['min_ano'] ?? $arrAnoMaximoMinimo['minimo'];


        if ($layout == 'sidebar') {
            include 'sidebar.php';
        }

        if ($layout == 'block') {
            include 'block.php';
        }

    }


    public function form($instance)
    {


        $title = isset($instance['title']) ? esc_attr($instance['title']) : '';
        $subtitle = isset($instance['subtitle']) ? esc_attr($instance['subtitle']) : '';
        $layout = isset($instance['layout']) ? esc_attr($instance['layout']) : '';
        ?>
        <p>
            <label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?></label>
            <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>"
                   name="<?php echo $this->get_field_name('title'); ?>" type="text"
                   value="<?php echo esc_attr($title); ?>"/>
        </p>

        <p>
            <label for="<?php echo $this->get_field_id('subtitle'); ?>"> Subtitulo</label>
            <input class="widefat" id="<?php echo $this->get_field_id('subtitle'); ?>"
                   name="<?php echo $this->get_field_name('subtitle'); ?>" type="text"
                   value="<?php echo esc_attr($subtitle); ?>"/>
        </p>


        <p>
            <label for="<?php echo $this->get_field_id('layout'); ?>">Layout</label>
            <select class="widefat"
                    id="<?php echo $this->get_field_id('layout'); ?>"
                    name="<?php echo $this->get_field_name('layout'); ?>"
            >
                <option value="block" <?= ($layout == 'block') ? 'selected' : ''; ?>>Bloco
                </option>
                <option value="sidebar" <?= ($layout == 'sidebar') ? 'selected' : ''; ?>>Sidebar
                </option>
            </select>
        </p>

        <?php
    }


    public function update($new_instance, $old_instance)
    {
        $instance = $old_instance;
        $instance['title'] = strip_tags($new_instance['title']);
        $instance['subtitle'] = strip_tags($new_instance['subtitle']);
        $instance['layout'] = strip_tags($new_instance['layout']);

        return $instance;

    }

    private function buscarFiliais($arrLocais)
    {
        if (!isset($arrLocais['locais'])) {
            return [];
        }

        return $arrLocais['locais'];
    }

    private function buscarMarcas($arrMarcas)
    {

        if (!isset($arrMarcas['marcas'])) {
            return [];
        }

        $arrReturn = [];
        foreach ($arrMarcas['marcas'] as $marca) {


            $slug = $marca['slug'];
            $data['id'] = $marca['term_id'];
            $data['name'] = $marca['name'];
            $data['slug'] = $slug;
            $data['url'] = get_term_link($marca['id'], 'marca');
            $data['img'] = get_term_meta($marca['id'], 'marca_img', true);


            if (empty($data['img'])) {
                continue;
            }
            $arrReturn[$slug] = $data;
        }


        return $arrReturn;
    }

    private function buscarModelos($arrModelos)
    {
        if (!isset($arrModelos['modelos'])) {
            return [];
        }

        $arrReturn = [];
        foreach ($arrModelos['modelos'] as $modelo) {

            $slug = $modelo['slug'];

            $data['id'] = $modelo['term_id'];
            $data['name'] = $modelo['name'];
            $data['slug'] = $slug;

            $arrReturn[$slug] = $data;
        }


        return $arrReturn;
    }

    private function buscarAnos($arrAnos)
    {

        if (!isset($arrAnos['ano'])) {
            return [];
        }


        return $arrAnos['ano'];
    }

    private function buscarAnoMaximoEMinimo()
    {

        $min_value = alt_get_min_year();
        $max_value = alt_get_max_year();

        return ['minimo' => $min_value, 'maximo' => $max_value];
    }

    private function buscarPrecoMaximoEMinimo()
    {

        $min_value = alt_get_min_price();
        $max_value = alt_get_max_price();

        return ['minimo' => $min_value, 'maximo' => $max_value];
    }

    private function listarVeiculos()
    {


        $args = array(
            'post_type' => 'veiculos',
            'posts_per_page' => '-1',
            'post_status' => 'publish',

        );
        $query_filtro = new WP_Query($args);


        while ($query_filtro->have_posts()) : $query_filtro->the_post();


            $objTipos = wp_get_object_terms(get_the_ID(), 'tipo');
            foreach ($objTipos as $objTipo) {
                $tipos['id'] = $objTipo->term_id;
                $tipos['name'] = $objTipo->name;
                $tipos['slug'] = $objTipo->slug;
                $arrReturn['tipos'][] = $tipos;
            }


            $objMarcas = wp_get_object_terms(get_the_ID(), 'marca');
            foreach ($objMarcas as $objMarca) {
                $marca['id'] = $objMarca->term_id;
                $marca['name'] = $objMarca->name;
                $marca['slug'] = $objMarca->slug;
                $arrReturn['marcas'][] = $marca;
            }


            $objCarroceria = wp_get_object_terms(get_the_ID(), 'carroceria');
            foreach ($objCarroceria as $objCarroceria) {

                $carroceria['id'] = $objCarroceria->term_id;
                $carroceria['name'] = $objCarroceria->name;
                $carroceria['slug'] = $objCarroceria->slug;
                $arrReturn['carroceria'][] = $carroceria;
            }


            $objModelos = wp_get_object_terms(get_the_ID(), 'modelo');
            foreach ($objModelos as $objMarca) {
                $marca['id'] = $objMarca->term_id;
                $marca['name'] = $objMarca->name;
                $marca['slug'] = $objMarca->slug;
                $arrReturn['modelos'][] = $marca;
            }


            $local = get_post_meta(get_the_ID(), 'alt_localizacao', true) ?? null;
            $ano = get_post_meta(get_the_ID(), 'alt_ano_mod', true) ?? null;
            $cambio = get_post_meta(get_the_ID(), 'alt_cambio', true) ?? null;


            $arrReturn['locais'][$local] = $local;
            $arrReturn['ano'][$ano] = $ano;
            $arrReturn['cambios'][$cambio] = $cambio;


        endwhile;


        //Realizar a ordenação
        $arrReturn['marcas'] = $this->unique_multidim_array($arrReturn['marcas'], 'id');
        $arrReturn['tipos'] = $this->unique_multidim_array($arrReturn['tipos'], 'id');
        $arrReturn['modelos'] = $this->unique_multidim_array($arrReturn['modelos'], 'id');
        $arrReturn['carroceria'] = $this->unique_multidim_array($arrReturn['carroceria'], 'id');


        wp_reset_postdata();

        return $arrReturn;
    }

    private function buscarTipos($arrVeiculos)
    {
        $categories = get_terms(['taxonomy' => 'carroceria', 'hide_empty' => true, 'orderby' => 'count', 'order' => 'DESC']);
        $arrRetorno = [];


        foreach ($categories as $key => $categoria) {

            $img = get_term_meta($categoria->term_id, 'carroceria_img', true);

            if (empty($img)) {
                continue;
            }

            $info['slug'] = $categoria->slug;
            $info['imagem'] = $img;
            $info['nome'] = $categoria->name;
            $info['total'] = $categoria->count;
            $info['slug'] = $categoria->slug;

            $arrRetorno[$categoria->slug] = $info;
        }


        return $arrRetorno;
    }


    private function get_min_year()
    {
        global $wpdb;

        $sql = "

        SELECT  MIN( meta_value+0 ) as minprice

        FROM {$wpdb->posts} 

        INNER JOIN {$wpdb->term_relationships} ON ({$wpdb->posts}.ID = {$wpdb->term_relationships}.object_id)

        INNER JOIN {$wpdb->postmeta} ON ({$wpdb->posts}.ID = {$wpdb->postmeta}.post_id) 

        WHERE  

        {$wpdb->posts}.post_type = 'product' 

        AND {$wpdb->posts}.post_status = 'publish' 

        AND {$wpdb->postmeta}.meta_key = 'alt_ano_mod'";

        return $wpdb->get_var($wpdb->prepare($sql, null));

    }


    private function get_max_year()
    {
        global $wpdb;

        $sql = "

        SELECT  MAX( meta_value+0 ) as minprice

        FROM {$wpdb->posts} 

        INNER JOIN {$wpdb->term_relationships} ON ({$wpdb->posts}.ID = {$wpdb->term_relationships}.object_id)

        INNER JOIN {$wpdb->postmeta} ON ({$wpdb->posts}.ID = {$wpdb->postmeta}.post_id) 

        WHERE  

        {$wpdb->posts}.post_type = 'product' 

        AND {$wpdb->posts}.post_status = 'publish' 

        AND {$wpdb->postmeta}.meta_key = 'alt_ano_mod'";

        return $wpdb->get_var($wpdb->prepare($sql, null));

    }

    private function unique_multidim_array($array, $key)
    {
        if (empty($array)) {
            return;
        }
        $temp_array = array();
        $i = 0;
        $key_array = array();

        foreach ($array as $val) {
            if (!in_array($val[$key], $key_array)) {
                $key_array[$i] = $val[$key];
                $temp_array[$i] = $val;
            }
            $i++;
        }
        return $temp_array;
    }
}
